@@ -12,6 +12,7 @@
 #include <sstream>
 #include <iomanip>
 #include <SDL.h>
+#include <SDL_endian.h>
 #include <string.h>
 #include <cassert>
 #include <sys/stat.h>
@@ -38,38 +39,29 @@
 #include "core/NstCrc32.hpp"
 #include "core/NstChecksum.hpp"
 #include "core/NstXml.hpp"
-#include "oss.h"
+#include "audio.h"
 #include "settings.h"
-#include "auxio.h"
+#include "fileio.h"
 #include "input.h"
 #include "controlconfig.h"
 #include "cheats.h"
 #include "seffect.h"
+#include "gtkui.h"
 #include "main.h"
 #include "GL/glu.h"
-
-#define NST_VERSION "1.43"
-
-extern "C" {
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 
+extern "C" {
 #include "callbacks.h"
 }
 
-#include "uihelp.h"
-
 using namespace Nes::Api;
 using namespace LinuxNst;
 
 // base class, all interfaces derives from this
 Emulator emulator;
 
-// forward declaration
-void SetupVideo();
-void SetupSound();
-void SetupInput();
-
 SDL_Surface *screen;
 
 static short lbuf[48000];
@@ -77,21 +69,21 @@ static long exholding[48000*2];
 
 static unsigned short keys[65536];
 
-static int updateok, playing = 0, cur_width, cur_Rwidth, cur_height, cur_Rheight, loaded = 0, framerate;
+static int updateok, playing = 0, cur_width, cur_height, loaded = 0, framerate;
 static int nst_quit = 0, nsf_mode = 0, state_save = 0, state_load = 0, movie_save = 0, movie_load = 0, movie_stop = 0;
 int schedule_stop = 0;
 static SDL_Joystick *joy[10];
 
-int xres;
-int yres;
+int cur_Rheight, cur_Rwidth;
 
 extern int lnxdrv_apimode;
-extern GtkWidget *mainwindow;
+extern GtkWidget *mainwindow, *statusbar;
 extern char windowid[24];
 
 static char savename[512], capname[512], gamebasename[512];
 static char caption[128];
 char rootname[512], lastarchname[512];
+char msgbuf[512];
 
 static InputDefT *ctl_defs;
 
@@ -477,7 +469,7 @@ void NstStopPlaying()
 		int i;
 
 		// kill any movie
-		auxio_do_movie_stop();
+		fileio_do_movie_stop();
 
 		// close video sanely
 		if (!nsf_mode)
@@ -551,13 +543,45 @@ std::string StrQuickSaveFile(int isvst)
 }
 
 void FlipFDSDisk() {
-	Fds fds( emulator );
+	Fds fds(emulator);
 
 	if (fds.CanChangeDiskSide()) {
 		fds.ChangeSide();
+		print_fds_info();
 	}
 }
 
+void SwitchFDSDisk() {
+	Fds fds(emulator);
+	
+	int currentdisk = fds.GetCurrentDisk();
+	
+	// If it's a multi-disk game, eject and insert the other disk
+	if (fds.GetNumDisks() > 1) {
+		fds.EjectDisk();
+		fds.InsertDisk(!currentdisk, 0);
+		print_fds_info();
+	}
+}
+
+void print_fds_info() {
+	Fds fds(emulator);
+
+	char* disk;
+	char* side;
+
+	fds.GetCurrentDisk() == 0 ? disk = "1" : disk = "2";
+	fds.GetCurrentDiskSide() == 0 ? side = "A" : side = "B";
+
+	snprintf(msgbuf, sizeof(msgbuf), "Fds: Disk %s Side %s", disk, side);
+	print_message(msgbuf);
+}
+
+void print_message(char* message) {
+	printf("%s\n", message);
+	gtk_statusbar_push(GTK_STATUSBAR(statusbar), 0, message);
+}
+
 // save state to memory slot
 static void QuickSave(int isvst)
 {
@@ -567,7 +591,8 @@ static void QuickSave(int isvst)
 	std::ofstream os(strFile.c_str());
 	// use "NO_COMPRESSION" to make it easier to hack save states
 	Nes::Result res = machine.SaveState(os, Nes::Api::Machine::USE_COMPRESSION);
-	printf("State Saved: %s\n", strFile.c_str());
+	snprintf(msgbuf, sizeof(msgbuf), "State Saved: %s", strFile.c_str());
+	print_message(msgbuf);
 }
 
 
@@ -578,14 +603,16 @@ static void QuickLoad(int isvst)
 	
 	struct stat qloadstat;
 	if (stat(strFile.c_str(), &qloadstat) == -1) {
-		printf("No State to Load\n");
+		snprintf(msgbuf, sizeof(msgbuf), "No State to Load");
+		print_message(msgbuf);
 		return;
 	}
 
 	Machine machine( emulator );
 	std::ifstream is(strFile.c_str());
 	Nes::Result res = machine.LoadState(is);
-	printf("State Loaded: %s\n", strFile.c_str());
+	snprintf(msgbuf, sizeof(msgbuf), "State Loaded: %s", strFile.c_str());
+	print_message(msgbuf);
 }
 
 
@@ -602,6 +629,8 @@ void NstPlayGame(void)
 		putenv(windowid);
 		NstStopPlaying();
 	}
+	
+	gtk_statusbar_push(GTK_STATUSBAR(statusbar), 0, "");
 
 	// initialization
 	SetupVideo();
@@ -650,9 +679,19 @@ void NstStopNsf(void)
 	memset(lbuf, 0, sizeof(lbuf));
 }
 
-void NstReset() {
-	Machine machine( emulator );
-	Fds fds( emulator );
+void NstSoftReset() {
+	Machine machine(emulator);
+	Fds fds(emulator);
+	machine.Reset(false);
+
+	// put the disk system back to disk 0 side 0
+	fds.EjectDisk();
+	fds.InsertDisk(0, 0);
+}
+
+void NstHardReset() {
+	Machine machine(emulator);
+	Fds fds(emulator);
 	machine.Reset(true);
 
 	// put the disk system back to disk 0 side 0
@@ -737,7 +776,7 @@ static void handle_input_event(Input::Controllers *controllers, InputEvtT inevt)
 		break;
 
 	case RESET:
-		NstReset();
+		NstSoftReset();
 		break;
 
 	case FLIP:
@@ -794,7 +833,6 @@ static void handle_input_event(Input::Controllers *controllers, InputEvtT inevt)
 	}
 }
 
-
 // match input event; if pind is not NULL, continue after it
 // on is set if the key/button is hit, clear if key is up/axis centered
 static const InputDefT *nst_match(const SDL_Event &evt, const InputDefT *pind, bool &on)
@@ -831,16 +869,8 @@ static const InputDefT *nst_match(const SDL_Event &evt, const InputDefT *pind, b
 					&& pind->evt.jbutton.which == evt.jbutton.which
 					&& pind->evt.jbutton.button == evt.jbutton.button;
 				on = (evt.jbutton.state == SDL_PRESSED);
-				//printf("%d", on);
 				break;
 
-			/* 
-			 * Below is the dirtiest hack I've ever seen/done. But it works.
-			 * The whole input system needs to be rewritten... it doesn't
-			 * look like hat switches were intended to be used originally.
-			 * I tried to do this cleanly, but after 4 days I said "fuck it".
-			 * Try to do it cleanly. I DARE YOU.
-			*/			
 			case SDL_JOYHATMOTION:
 					match = pind->evt.type == SDL_JOYHATMOTION
 						&& pind->evt.jhat.which == evt.jhat.which
@@ -878,7 +908,6 @@ static const InputDefT *nst_match(const SDL_Event &evt, const InputDefT *pind, b
 						return pind;
 					}
 				break;
-				// End of dirty hack
 
 			case SDL_JOYAXISMOTION:
 			{
@@ -961,7 +990,7 @@ static void NST_CALLBACK DoFileIO(void *userData, User::File& file)
 		case User::File::LOAD_ROM:
 			wcstombs(mbstr, file.GetName(), 511);
 
-			if (auxio_load_archive(lastarchname, &compbuffer, &compsize, &compoffset, (const char *)mbstr))
+			if (fileio_load_archive(lastarchname, &compbuffer, &compsize, &compoffset, (const char *)mbstr))
 			{
 				file.SetContent((const void*)&compbuffer[compoffset], (unsigned long int)compsize);
 
@@ -977,7 +1006,7 @@ static void NST_CALLBACK DoFileIO(void *userData, User::File& file)
 		case User::File::LOAD_SAMPLE_AEROBICS_STUDIO:
 			wcstombs(mbstr, file.GetName(), 511);
 
-			if (auxio_load_archive(lastarchname, &compbuffer, &compsize, &compoffset, (const char *)mbstr))
+			if (fileio_load_archive(lastarchname, &compbuffer, &compsize, &compoffset, (const char *)mbstr))
 			{
 				int chan, bits, rate;
 
@@ -1043,14 +1072,14 @@ static void NST_CALLBACK DoFileIO(void *userData, User::File& file)
 		{
 			char fdsname[512];
 
-			sprintf(fdsname, "%s.ups", rootname);
+			snprintf(fdsname, sizeof(fdsname), "%s.ups", rootname);
 			
 			std::ifstream batteryFile( fdsname, std::ifstream::in|std::ifstream::binary );
 
 			// no ups, look for ips
 			if (!batteryFile.is_open())
 			{
-				sprintf(fdsname, "%s.ips", rootname);
+				snprintf(fdsname, sizeof(fdsname), "%s.ips", rootname);
 
 				std::ifstream batteryFile( fdsname, std::ifstream::in|std::ifstream::binary );
 
@@ -1071,7 +1100,7 @@ static void NST_CALLBACK DoFileIO(void *userData, User::File& file)
 		{
 			char fdsname[512];
 
-			sprintf(fdsname, "%s.ups", rootname);
+			snprintf(fdsname, sizeof(fdsname), "%s.ups", rootname);
 
 			std::ofstream fdsFile( fdsname, std::ifstream::out|std::ifstream::binary );
 
@@ -1110,8 +1139,8 @@ int main(int argc, char *argv[])
 		
 		// make sure the output directory exists
 		home = getenv("HOME");
-		sprintf(dirname, "%s/.nestopia/", home);
-		sprintf(savedirname, "%ssave/", dirname);
+		snprintf(dirname, sizeof(dirname), "%s/.nestopia/", home);
+		snprintf(savedirname, sizeof(dirname), "%ssave/", dirname);
 		mkdir(dirname, 0700);
 		mkdir(savedirname, 0700);
 		create_input_file();
@@ -1127,7 +1156,7 @@ int main(int argc, char *argv[])
 	playing = 0;
 	intbuffer = NULL;
 
-	auxio_init();
+	fileio_init();
 
 	sSettings = new Settings;
 	sCheatMgr = new CheatMgr;
@@ -1147,10 +1176,10 @@ int main(int argc, char *argv[])
 	User::logCallback.Set( DoLog, userData );
 
 	// try to load the FDS BIOS
-	auxio_set_fds_bios();
+	fileio_set_fds_bios();
 
 	// and the NST database
-	auxio_load_db();
+	fileio_load_db();
 
 	// attempt to load and autostart a file specified on the commandline
 	if (argc > 1)
@@ -1221,7 +1250,7 @@ int main(int argc, char *argv[])
 			if (state_save)
 			{
 				kill_video_if_fs();
-				auxio_do_state_save();
+				fileio_do_state_save();
 				state_save = 0;
 				cleanup_after_io();
 			}
@@ -1229,7 +1258,7 @@ int main(int argc, char *argv[])
 			if (state_load)
 			{
 				kill_video_if_fs();
-				auxio_do_state_load();
+				fileio_do_state_load();
 				state_load = 0;
 				cleanup_after_io();
 			}
@@ -1237,7 +1266,7 @@ int main(int argc, char *argv[])
 			if (movie_load)
 			{
 				kill_video_if_fs();
-				auxio_do_movie_load();
+				fileio_do_movie_load();
 				movie_load = 0;
 				cleanup_after_io();
 			}
@@ -1245,7 +1274,7 @@ int main(int argc, char *argv[])
 			if (movie_save)
 			{
 				kill_video_if_fs();
-				auxio_do_movie_save();
+				fileio_do_movie_save();
 				movie_load = 0;
 				cleanup_after_io();
 			}
@@ -1253,7 +1282,7 @@ int main(int argc, char *argv[])
 			if (movie_stop)
 			{
 				movie_stop = 0;
-				auxio_do_movie_stop();
+				fileio_do_movie_stop();
 			}
 
 			if (schedule_stop)
@@ -1269,7 +1298,7 @@ int main(int argc, char *argv[])
 
 	nst_unload();
 
-	auxio_shutdown();
+	fileio_shutdown();
 
 	delete sSettings;
 	delete sCheatMgr;
@@ -1340,18 +1369,20 @@ void get_screen_res() {
 			sSettings->GetTvAspect() == TRUE ? cur_Rwidth = TV_WIDTH * scalefactor : cur_Rwidth = cur_width;
 			cur_height = cur_Rheight = Video::Output::HEIGHT * scalefactor;
 			break;
-			
+
 		case 4: // 2xSaI
 			cur_width = Video::Output::WIDTH * 2;
 			sSettings->GetTvAspect() == TRUE ? cur_Rwidth = TV_WIDTH * scalefactor : cur_Rwidth = Video::Output::WIDTH * scalefactor;
 			cur_height = Video::Output::HEIGHT * 2;
 			cur_Rheight = Video::Output::HEIGHT * scalefactor;
 			break;
-	}
 
-	//This is somewhat dirty, but it makes it easier to pass the data to other functions
-	xres = cur_Rwidth;
-	yres = cur_Rheight;
+		case 5: // scale xBR
+			cur_width = Video::Output::WIDTH * scalefactor;
+			sSettings->GetTvAspect() == TRUE ? cur_Rwidth = TV_WIDTH * scalefactor : cur_Rwidth = cur_width;
+			cur_height = cur_Rheight = Video::Output::HEIGHT * scalefactor;
+			break;
+	}
 }
 
 void SetupVideo()
@@ -1476,6 +1507,27 @@ void SetupVideo()
 		case 4: // 2xSaI
 			filter = Video::RenderState::FILTER_2XSAI;
 			break;
+
+		case 5: // xBR
+			switch (scalefactor)
+			{
+				case 2:
+					filter = Video::RenderState::FILTER_2XBR;
+					break;
+
+				case 3:
+					filter = Video::RenderState::FILTER_3XBR;
+					break;
+
+				case 4:
+					filter = Video::RenderState::FILTER_4XBR;
+					break;
+
+				default:
+					filter = Video::RenderState::FILTER_NONE;
+					break;
+			}
+			break;
 	}
 
 	int eFlags = SDL_HWSURFACE;
@@ -1521,9 +1573,15 @@ void SetupVideo()
 		opengl_init_structures();
 
 		renderState.bits.count = 32;
+	#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+		renderState.bits.mask.r = 0x000000ff;
+		renderState.bits.mask.g = 0xff000000;
+		renderState.bits.mask.b = 0x00ff0000;
+	#else
 		renderState.bits.mask.r = 0x00ff0000;
 		renderState.bits.mask.g = 0x0000ff00;
 		renderState.bits.mask.b = 0x000000ff;
+	#endif
 	}
 	else
 	{
@@ -1569,7 +1627,14 @@ void SetupVideo()
 			video.SetColorFringing(Video::DEFAULT_COLOR_FRINGING_RGB);
 			break;
 	}
-			     
+	
+	if (sSettings->GetScale() == 5) {
+		video.SetCornerRounding(sSettings->GetCornerRounding()); // 0 = none, 1 = some, 2 = all
+		video.SetBlend(sSettings->GetBlendPix()); // boolean
+	}
+	
+	video.ClearFilterUpdateFlag();
+
 	// set the render state
 	if (NES_FAILED(video.SetRenderState( renderState )))
 	{
@@ -1628,7 +1693,7 @@ void configure_savename( const char* filename )
 	char savedir[1024], *homedir;
 
 	homedir = getenv("HOME");
-	sprintf(savedir, "%s/.nestopia/save/", homedir);
+	snprintf(savedir, sizeof(savedir), "%s/.nestopia/save/", homedir);
 
 	strcpy(savename, filename);
 
@@ -1660,7 +1725,7 @@ void configure_savename( const char* filename )
 	strcpy(savename, savedir);
 	
 	// also generate the window caption
-	sprintf(caption, "Nestopia");
+	snprintf(caption, sizeof(caption), "Nestopia");
 
 	strcpy(rootname, savename);
 	strcat(savename, ".sav");
@@ -1728,7 +1793,7 @@ void NstLoadGame(const char* filename)
 
 	// check if it's an archive
 	wascomp = 0;
-	if (auxio_load_archive(filename, &compbuffer, &compsize, &compoffset, NULL, gamename))
+	if (fileio_load_archive(filename, &compbuffer, &compsize, &compoffset, NULL, gamename))
 	{
 		std::istrstream file((char *)compbuffer+compoffset, compsize);
 		wascomp = 1;
@@ -1829,7 +1894,8 @@ void NstLoadGame(const char* filename)
 				break;
 
 			case Nes::RESULT_ERR_MISSING_BIOS:
-				std::cout << "Can't find disksys.rom for FDS game\n";
+				snprintf(msgbuf, sizeof(msgbuf), "FDS games require the FDS BIOS.\nIt should be located at ~/.nestopia/disksys.rom");
+				create_messagewindow(msgbuf);
 				break;
 
 			default:
@@ -1881,6 +1947,7 @@ void NstLoadGame(const char* filename)
 			Fds fds( emulator );
 
 			fds.InsertDisk(0, 0);
+			print_fds_info();
 		}
 	}
 
